@php
	use App\Models\Vehicle;
	/** @var Vehicle $vehicle */
@endphp
<x-layout title="{{ __('Vehicles') }}">
	<x-slot:breadcrumb>
		{{ Breadcrumbs::render('vehicles') }}
	</x-slot:breadcrumb>
	<x-slot name="header">
		<h2>{{ __('Vehicles') }}</h2>
		<x-searchbar-select2 post="{{ route('vehicles') }}" inputClasses="w-[20rem]"
		                     autocomplete="{{ route('vehicles.autocomplete') }}"
		                     placeholder="Select Vehicle" templateResult="true"
		                     templateSelection="true" text="item.vehicle_name">
			<x-select-dropdown-hover alpineVariable="vehicleClass" placeholder="Pick a Vehicle Class..."
			                         :enums="$vehicleClasses" functionCall="getDisplayName"/>
			<x-select-dropdown-hover alpineVariable="manufacturer" placeholder="Pick a Manufacturer..."
			                         :enums="$manufacturers" functionCall="getDisplayName"/>
			<x-select-dropdown-hover alpineVariable="fuelType" placeholder="Pick a Fuel Type..."
			                         :enums="$fuelTypes" functionCall="getDisplayName"/>
			<x-select-dropdown-hover alpineVariable="type" placeholder="Pick a Vehicle Type..."
			                         :enums="$types" functionCall="getDisplayName"/>
			<x-select-dropdown-hover alpineVariable="listMode" :enums="['gridCards', 'tableHorizontal', 'tableVertical']" placeholder="Pick a display mode for the list..." default="gridCards"/>
		</x-searchbar-select2>
	</x-slot>
	@php
		$fields = explode(',', env('FIELDS', ''));
        $headerValues = [
            'image' => ['type' => 'string', 'value' => __('Photo'),],
			'name' => ['type' => 'string', 'value' => __('Name')],
			'manufacturer' => ['type' => 'string', 'value' => __('Manufacturer')],
			'class' => ['type' => 'string', 'value' => __('Class')],
			'storage' => ['type' => 'string', 'value' => __('Storage')],
			'fuel_type' => ['type' => 'string', 'value' => __('Fuel Type')],
			'fuel_volume' => ['type' => 'string', 'value' => __('Fuel Volume')],
			'seats' => ['type' => 'string', 'value' => __('Seats')],
			'top_speed' => ['type' => 'string', 'value' => __('Top Speed')],
			'price' => ['type' => 'string', 'value' => __('Price')],
        ];
		$values = [];
        foreach ($fields as $field) {
            $values[0][$field] = $headerValues[$field];
        }

        $index = 0;
		foreach ($vehicles as $key => $vehicle) {
            $index++;
            $fieldValues = [
				'image' => ['type' => 'image', 'value' => 'https://cdn.jsdelivr.net/gh/Stuyk/gtav-image-archive/vehicles/' . $vehicle->getModel() . '.webp'],
				'name' => ['type' => 'string', 'value' => $vehicle->getDisplayName()],
				'manufacturer' => ['type' => 'string', 'value' => $manufacturers->firstWhere('name', '=', $vehicle->getManufacturer())->getDisplayName()],
				'class' => ['type' => 'string', 'value' => $vehicleClasses->firstWhere('name', '=', strtolower($vehicle->getClass()))->getDisplayName()],
				'storage' => ['type' => 'int', 'value' => $vehicle->getStorage()],
				'fuel_type' => ['type' => 'string', 'value' => $fuelTypes->firstWhere('name', '=', $vehicle->getFuelType())->getDisplayName()],
				'fuel_volume' => ['type' => 'int', 'value' => $vehicle->getFuelTankVolume(), 'unit' => env('FUEL_VOLUME_UNIT', 'l')],
				'seats' => ['type' => 'int', 'value' => $vehicle->getSeats()],
				'top_speed' => ['type' => 'int', 'value' => $vehicle->getTopSpeed(), 'unit' => env('SPEED_UNIT', 'km/h')],
				'price' => ['type' => 'int', 'value' => $vehicle->getPrice(), 'unit' => env('CURRENCY', '$')],
			];

			foreach ($fields as $field) {
				$values[$index][$field] = $fieldValues[$field];
            }
	}
	@endphp
	@switch(app('request')->input('listMode') ?? env('DEFAULT_DISPLAY_MODE'))
		@case('tableHorizontal')
		@case('tableVertical')
			<x-table.table :mode="app('request')->input('listMode') ?? env('DEFAULT_DISPLAY_MODE')" :values="$values" />
			@break
		@default
			<x-grid>
				@foreach($vehicles as $key => $vehicle)
					<x-vehicles.card :vehicle="$vehicle"
					                 :brand="$manufacturers->firstWhere('name', '=', $vehicle->getManufacturer())->getDisplayName()"
					                 :class="$vehicleClasses->firstWhere('name', '=', strtolower($vehicle->getClass()))->getDisplayName()"
					                 :fuel-type="$fuelTypes->firstWhere('name', '=', $vehicle->getFuelType())->getDisplayName()"/>
				@endforeach
			</x-grid>
	@endswitch
	{{ $vehicles->links() }}
</x-layout>